@@ -14,6 +14,7 @@ from .charge import MoleculeChargeConstraints
 from .resp import RespCharges
 from .orientation import Orientation
 from .constraint import ESPSurfaceConstraintMatrix
+from .utils import require_package
 
 logger = logging.getLogger(__name__)
 
@@ -176,6 +177,7 @@ class Job(base.Model):
 
     def _compute_esp(self, orientation):
         """Compute the grid and ESP for an orientation with the job's grid options"""
+        require_package("psi4")
         if orientation.grid is None:
             orientation.compute_grid(grid_options=self.grid_options)
         orientation.compute_esp()
@@ -184,6 +186,7 @@ class Job(base.Model):
 
     def _try_compute_esp(self, orientation):
         """Wrap ESP computation in a try/except to defer errors"""
+        require_package("psi4")
         try:
             return self._compute_esp(orientation)
         except BaseException as e:
@@ -191,12 +194,18 @@ class Job(base.Model):
 
     def run(self, client=None, update_molecules: bool = True) -> np.ndarray:
         """Run the whole job"""
+        # die early on failure to import
+        # we can't decorate the function because pickle is sad
+        require_package("psi4")
+
         self.generate_conformers()
         self.optimize_geometries(client=client)
         self.generate_orientations()
         return self.compute_esps_and_charges(client=client, update_molecules=update_molecules)
 
     def compute_esps_and_charges(self, client=None, update_molecules: bool = True) -> np.ndarray:
+        require_package("psi4")
+
         self.compute_orientation_energies(client=client)
         self.compute_esps()
         self.compute_charges(update_molecules=update_molecules)
@@ -214,14 +223,14 @@ class Job(base.Model):
             )
             for mol in self.molecules
         ]
-        a_mol = scipy.linalg.block_diag(*[mat.a for mat in matrices])
-        a_row = scipy.linalg.block_diag(*[np.ones(mat.b.shape[0]) for mat in matrices])
+        a_mol = scipy.linalg.block_diag(*[mat.coefficient_matrix for mat in matrices])
+        a_row = scipy.linalg.block_diag(*[np.ones(mat.constant_vector.shape[0]) for mat in matrices])
         a_zeros = np.zeros((a_row.shape[0], a_row.shape[0]))
         a_block = np.bmat([[a_mol, a_row.T], [a_row, a_zeros]])
 
-        b_block = np.concatenate([mat.b for mat in matrices]
+        b_block = np.concatenate([mat.constant_vector for mat in matrices]
                                  + [[mol.charge for mol in self.molecules]])
-        return ESPSurfaceConstraintMatrix.from_a_and_b(a_block, b_block)
+        return ESPSurfaceConstraintMatrix.from_coefficient_matrix(a_block, b_block)
 
     def generate_molecule_charge_constraints(self) -> MoleculeChargeConstraints:
         """
@@ -245,7 +254,7 @@ class Job(base.Model):
 
         self.stage_1_charges = RespCharges(charge_constraints=stage_1_constraints,
                                            surface_constraints=surface_constraints,
-                                           resp_a=self.resp_options.resp_a1,
+                                           restraint_height=self.resp_options.restraint_height_stage_1,
                                            **self.resp_options._base_kwargs)
         self.stage_1_charges.solve()
 
@@ -253,7 +262,7 @@ class Job(base.Model):
             stage_2_constraints.add_constraints_from_charges(self.stage_1_charges._charges)
             self.stage_2_charges = RespCharges(charge_constraints=stage_2_constraints,
                                                surface_constraints=surface_constraints,
-                                               resp_a=self.resp_options.resp_a2,
+                                               restraint_height=self.resp_options.restraint_height_stage_2,
                                                **self.resp_options._base_kwargs)
             self.stage_2_charges.solve()
 
