@@ -8,6 +8,7 @@ from .constraint import ESPSurfaceConstraintMatrix
 from .moleculebase import BaseMolecule
 from .grid import GridOptions
 from .qcutils import QCWaveFunction
+from .utils import require_package
 
 
 class Orientation(BaseMolecule):
@@ -40,6 +41,7 @@ class Orientation(BaseMolecule):
         self.grid = grid_options.generate_grid(self.qcmol)
 
     def compute_esp(self):
+        require_package("psi4")
         from . import psi4utils
         self.esp = psi4utils.compute_esp(self.qc_wavefunction, self.grid)
         return self.esp
@@ -81,6 +83,6 @@ class Orientation(BaseMolecule):
         a = np.einsum("ij, ik->jk", r_inv, r_inv)
         b = np.einsum("i, ij->j", self.esp, r_inv)
 
-        matrix = ESPSurfaceConstraintMatrix.from_a_and_b(a, b)
+        matrix = ESPSurfaceConstraintMatrix.from_coefficient_matrix(a, b)
         self._constraint_matrix = matrix
         return matrix
